@@ -23,7 +23,11 @@ import sys, os
 
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
-extensions = ['sphinx.ext.imgmath']
+on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
+if on_rtd:
+    extensions = ['sphinx.ext.mathjax']
+else:
+    extensions = ['sphinx.ext.imgmath']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['.templates']
@@ -36,15 +40,15 @@ master_doc = 'index'
 
 # General substitutions.
 project = "CVXOPT User's Guide"
-copyright = '2004-2016, M.S. Andersen, J. Dahl, L. Vandenberghe'
+copyright = '2004-2019, M.S. Andersen, J. Dahl, L. Vandenberghe'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '1.1.9'
+version = '1.2.3'
 # The full version, including alpha/beta/rc tags.
-release = '1.1.9'
+release = '1.2.3'
 
 
 # There are two options for replacing |today|: either, you set today to some
