@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 M. Andersen and L. Vandenberghe.
+ * Copyright 2012-2019 M. Andersen and L. Vandenberghe.
  * Copyright 2010-2011 L. Vandenberghe.
  * Copyright 2004-2009 J. Dahl and L. Vandenberghe.
  *
@@ -26,7 +26,13 @@
 #include "cholmod.h"
 #include "complex.h"
 
-const int E_SIZE[] = { sizeof(int_t), sizeof(double), sizeof(double complex) };
+#ifndef _MSC_VER
+typedef double complex complex_t;
+#else
+typedef _Dcomplex complex_t;
+#endif
+
+const int E_SIZE[] = {sizeof(int_t), sizeof(double), sizeof(complex_t)};
 
 /* defined in pyconfig.h */
 #if (SIZEOF_INT < SIZEOF_SIZE_T)
@@ -57,7 +63,7 @@ PyDoc_STRVAR(cholmod__doc__, "Interface to the CHOLMOD library.\n\n"
 "    the user-provided ordering and the AMD ordering are compared, and \n"
 "    the best ordering is used. (If the user does not provide an \n"
 "    ordering the AMD ordering is used.)  If nmethods is equal to 1, \n"
-"    the user-provided ordering is used. (In this case, the user must \n" 
+"    the user-provided ordering is used. (In this case, the user must \n"
 "    provide an ordering.   If nmethods is equal to 0, the user-provided\n"
 "    ordering (if any), the AMD ordering, and (if installed during the \n"
 "    the CHOLMOD installation) a number of other orderings are compared,\n"
@@ -78,7 +84,7 @@ static int set_options(void)
     PyObject *param, *key, *value;
     char err_str[100];
 #if PY_MAJOR_VERSION < 3
-    char *keystr; 
+    char *keystr;
 #endif
 
     CHOL(defaults)(&Common);
@@ -162,7 +168,7 @@ static cholmod_sparse *pack(spmatrix *A, char uplo)
                 if (SP_ID(A) == DOUBLE)
                     ((double *)B->x)[cnt] = SP_VALD(A)[k];
                 else
-                    ((double complex *)B->x)[cnt] = SP_VALZ(A)[k];
+                    ((complex_t *)B->x)[cnt] = SP_VALZ(A)[k];
                 ((int_t *)B->p)[j+1]++;
                 ((int_t *)B->i)[cnt++] = SP_ROW(A)[k];
 	    }
@@ -183,7 +189,7 @@ static cholmod_sparse *pack(spmatrix *A, char uplo)
                 if (SP_ID(A) == DOUBLE)
                     ((double *)B->x)[cnt] = SP_VALD(A)[k];
                 else
-                    ((double complex *)B->x)[cnt] = SP_VALZ(A)[k];
+                    ((complex_t *)B->x)[cnt] = SP_VALZ(A)[k];
             ((int_t *)B->p)[j+1]++;
             ((int_t *)B->i)[cnt++] = SP_ROW(A)[k];
         }
@@ -222,7 +228,7 @@ static void free_matrix(cholmod_sparse *A)
 #if PY_MAJOR_VERSION >= 3
 static void cvxopt_free_cholmod_factor(void *L)
 {
-   void *Lptr = PyCapsule_GetPointer(L, PyCapsule_GetName(L));   
+   void *Lptr = PyCapsule_GetPointer(L, PyCapsule_GetName(L));
    CHOL(free_factor) ((cholmod_factor **) &Lptr, &Common);
 }
 #else
@@ -308,13 +314,13 @@ static PyObject* symbolic(PyObject *self, PyObject *args,
         }
     }
 #if PY_MAJOR_VERSION >= 3
-    return (PyObject *) PyCapsule_New((void *) L, SP_ID(A)==DOUBLE ?  
+    return (PyObject *) PyCapsule_New((void *) L, SP_ID(A)==DOUBLE ?
         (uplo == 'L' ?  "CHOLMOD FACTOR D L" : "CHOLMOD FACTOR D U") :
         (uplo == 'L' ?  "CHOLMOD FACTOR Z L" : "CHOLMOD FACTOR Z U"),
-        (PyCapsule_Destructor) &cvxopt_free_cholmod_factor); 
+        (PyCapsule_Destructor) &cvxopt_free_cholmod_factor);
 #else
     return (PyObject *) PyCObject_FromVoidPtrAndDesc((void *) L,
-        SP_ID(A)==DOUBLE ?  
+        SP_ID(A)==DOUBLE ?
         (uplo == 'L' ?  "CHOLMOD FACTOR D L" : "CHOLMOD FACTOR D U") :
         (uplo == 'L' ?  "CHOLMOD FACTOR Z L" : "CHOLMOD FACTOR Z U"),
 	cvxopt_free_cholmod_factor);
@@ -346,7 +352,7 @@ static char doc_numeric[] =
     "          uplo='U', then only the upper triangular part is used.\n"
     "\n"
     "F         symbolic factorization computed by cholmod.symbolic\n"
-    "          applied to a matrix with the same sparsity patter and\n"
+    "          applied to a matrix with the same sparsity pattern and\n"
     "          type as A.  After a successful call, F contains the\n"
     "          numeric factorization.";
 
@@ -358,9 +364,9 @@ static PyObject* numeric(PyObject *self, PyObject *args)
     cholmod_sparse *Ac = NULL;
     char uplo;
 #if PY_MAJOR_VERSION >= 3
-    const char *descr; 
+    const char *descr;
 #else
-    char *descr; 
+    char *descr;
 #endif
 
     if (!set_options()) return NULL;
@@ -631,7 +637,7 @@ static PyObject* spsolve(PyObject *self, PyObject *args,
         ((int_t*)Xc->p)[Xc->ncol], (L->xtype == CHOLMOD_REAL ? DOUBLE :
         COMPLEX)))) {
         CHOL(free_sparse)(&Xc, &Common);
-        return PyErr_NoMemory();
+        return NULL;
     }
     memcpy(SP_COL(X), Xc->p, (Xc->ncol+1)*sizeof(int_t));
     memcpy(SP_ROW(X), Xc->i, ((int_t *)Xc->p)[Xc->ncol]*sizeof(int_t));
@@ -675,7 +681,7 @@ static PyObject* linsolve(PyObject *self, PyObject *args,
 {
     spmatrix *A;
     matrix *B, *P=NULL;
-    int i, n, nnz, oB=0, ldB=0, nrhs=-1;
+    int i, n, oB=0, ldB=0, nrhs=-1;
     cholmod_sparse *Ac=NULL;
     cholmod_factor *L=NULL;
     cholmod_dense *x=NULL, *b=NULL;
@@ -700,7 +706,6 @@ static PyObject* linsolve(PyObject *self, PyObject *args,
     if (!SpMatrix_Check(A) || SP_NROWS(A) != SP_NCOLS(A))
         PY_ERR_TYPE("A is not a sparse matrix");
     n = SP_NROWS(A);
-    nnz = SP_NNZ(A);
 
     if (!Matrix_Check(B) || MAT_ID(B) != SP_ID(A))
         PY_ERR_TYPE("B must be a dense matrix of the same numerical "
@@ -828,7 +833,7 @@ static PyObject* splinsolve(PyObject *self, PyObject *args,
 {
     spmatrix *A, *B, *X;
     matrix *P=NULL;
-    int n, nnz;
+    int n;
     cholmod_sparse *Ac=NULL, *Bc=NULL, *Xc=NULL;
     cholmod_factor *L=NULL;
 #if PY_MAJOR_VERSION >= 3
@@ -850,7 +855,6 @@ static PyObject* splinsolve(PyObject *self, PyObject *args,
     if (!SpMatrix_Check(A) || SP_NROWS(A) != SP_NCOLS(A))
         PY_ERR_TYPE("A is not a square sparse matrix");
     n = SP_NROWS(A);
-    nnz = SP_NNZ(A);
 
     if (!SpMatrix_Check(B) || SP_ID(A) != SP_ID(B))
         PY_ERR_TYPE("B must be a sparse matrix of the same type as A");
@@ -928,7 +932,7 @@ static PyObject* splinsolve(PyObject *self, PyObject *args,
     if (!(X = SpMatrix_New(Xc->nrow, Xc->ncol,
         ((int_t*)Xc->p)[Xc->ncol], SP_ID(A)))) {
         CHOL(free_sparse)(&Xc, &Common);
-        return PyErr_NoMemory();
+        return NULL;
     }
     memcpy(SP_COL(X), (int_t *) Xc->p, (Xc->ncol+1)*sizeof(int_t));
     memcpy(SP_ROW(X), (int_t *) Xc->i,
@@ -954,7 +958,7 @@ static char doc_diag[] =
     "          cholmod.numeric computed with options['supernodal'] = 2";
 
 extern void dcopy_(int *n, double *x, int *incx, double *y, int *incy);
-extern void zcopy_(int *n, double complex *x, int *incx, double complex *y, int *incy);
+extern void zcopy_(int *n, complex_t *x, int *incx, complex_t *y, int *incy);
 
 static PyObject* diag(PyObject *self, PyObject *args)
 {
@@ -991,7 +995,7 @@ static PyObject* diag(PyObject *self, PyObject *args)
         PY_ERR(PyExc_ValueError, "F must be a nonsingular supernodal "
             "Cholesky factor");
     if (!(d = Matrix_New(L->n,1,L->xtype == CHOLMOD_REAL ? DOUBLE :
-        COMPLEX))) return PyErr_NoMemory();
+			 COMPLEX))) return NULL;
 
     strt = 0;
     for (k=0; k<L->nsuper; k++){
@@ -1007,7 +1011,7 @@ static PyObject* diag(PyObject *self, PyObject *args)
 	    dcopy_(&ncols, ((double *) L->x) + ((int_t *) L->px)[k],
                 &incy, MAT_BUFD(d)+strt, &incx);
         else
-	    zcopy_(&ncols, ((double complex *) L->x) + ((int_t *) L->px)[k],
+	    zcopy_(&ncols, ((complex_t *) L->x) + ((int_t *) L->px)[k],
                 &incy, MAT_BUFZ(d)+strt, &incx);
         strt += ncols;
     }
@@ -1054,7 +1058,7 @@ static PyObject* getfactor(PyObject *self, PyObject *args)
        (Ls->xtype == CHOLMOD_REAL ? DOUBLE : COMPLEX));
     if (!ret) {
         CHOL(free_sparse)(&Ls, &Common);
-        return PyErr_NoMemory();
+        return NULL;
     }
 
     memcpy(SP_COL(ret), Ls->p, (Ls->ncol+1)*sizeof(int_t));
@@ -1106,7 +1110,7 @@ PyMODINIT_FUNC PyInit_cholmod(void)
     return cholmod_module;
 }
 
-#else 
+#else
 
 PyMODINIT_FUNC initcholmod(void)
 {
