@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 M. Andersen and L. Vandenberghe.
+ * Copyright 2012-2019 M. Andersen and L. Vandenberghe.
  * Copyright 2010-2011 L. Vandenberghe.
  * Copyright 2004-2009 J. Dahl and L. Vandenberghe.
  *
@@ -25,6 +25,7 @@
 #include "cvxopt.h"
 #include "misc.h"
 #include <complexobject.h>
+#include <limits.h>
 
 /* prototyping and forward declarations */
 extern void (*axpy[])(int *, number *, void *, int *, void *, int *) ;
@@ -66,14 +67,26 @@ static PyObject * matrix_iter(matrix *) ;
 
 
 void dscal_(int *, double *, double *, int *) ;
+#ifndef _MSC_VER
 void zscal_(int *, double complex *, double complex *, int *) ;
+#else
+void zscal_(int *, _Dcomplex *, _Dcomplex *, int *) ;
+#endif
 void daxpy_(int *, double *, double *, int *, double *, int *) ;
+#ifndef _MSC_VER
 void zaxpy_(int *, double complex *, double complex *, int *, double complex *, int *) ;
+#else
+void zaxpy_(int *, _Dcomplex *, _Dcomplex *, int *, _Dcomplex *, int *) ;
+#endif
 void dgemm_(char *, char *, int *, int *, int *, double *, double *,
     int *, double *, int *, double *, double *, int *) ;
+#ifndef _MSC_VER
 void zgemm_(char *, char *, int *, int *, int *, double complex *, double complex *,
     int *, double complex *, int *, double complex *, double complex *, int *) ;
-
+#else
+void zgemm_(char *, char *, int *, int *, int *, _Dcomplex *, _Dcomplex *,
+    int *, _Dcomplex *, int *, _Dcomplex *, _Dcomplex *, int *) ;
+#endif
 
 
 static const char err_mtx_list2matrix[][35] =
@@ -136,9 +149,14 @@ matrix * Matrix_New(int nrows, int ncols, int id)
     PyErr_BadInternalCall();
     return NULL;
   }
-
-  if (!(a = (matrix *)matrix_tp.tp_alloc(&matrix_tp, 0)))
+  else if (ncols > 0 && nrows > (INT_MAX/ncols)) {
+    PyErr_SetString(PyExc_OverflowError, "number of elements exceeds INT_MAX");
     return NULL;
+  }
+  else if (!(a = (matrix *)matrix_tp.tp_alloc(&matrix_tp, 0))) {
+    PyErr_NoMemory();
+    return NULL;
+  }
 
   a->id = id; a->nrows = nrows; a->ncols = ncols;
   if ((a->buffer = calloc(nrows*ncols,E_SIZE[id])))
@@ -165,8 +183,8 @@ matrix *Matrix_NewFromMatrix(matrix *src, int id)
   if (PY_NUMBER((PyObject *)src))
     return Matrix_NewFromNumber(1, 1, id, src, 1);
 
-  if (!(a = Matrix_New(src->nrows, src->ncols, id)))
-    return (matrix *)PyErr_NoMemory();
+  if (!(a = Matrix_New(src->nrows, src->ncols, id))) 
+    return NULL;
 
   if (convert_mtx(src, a->buffer, id)) {
     Py_DECREF(a); PY_ERR_TYPE("illegal type conversion");
@@ -226,7 +244,7 @@ matrix *Matrix_NewFromPyBuffer(PyObject *obj, int id, int *ndim)
  if (!a) {
    PyBuffer_Release(view); 
    free(view);
-   return (matrix *)PyErr_NoMemory();
+   return NULL;
  }
 
  int i, j, cnt;
@@ -261,16 +279,31 @@ matrix *Matrix_NewFromPyBuffer(PyObject *obj, int id, int *ndim)
      case COMPLEX:
        switch (src_id) {
        case INT:
+#ifndef _MSC_VER
 	 if (int_to_int_t)
 	   n.z = *(int *)((unsigned char*)view->buf + i*view->strides[0]+j*view->strides[1]);
 	 else
 	   n.z = *(int_t *)((unsigned char*)view->buf + i*view->strides[0]+j*view->strides[1]);
+#else
+	 if (int_to_int_t)
+	   n.z = _Cbuild((double) *(int *)((unsigned char*)view->buf + i*view->strides[0]+j*view->strides[1]),0.0);
+	 else
+	   n.z = _Cbuild((double) *(int_t *)((unsigned char*)view->buf + i*view->strides[0]+j*view->strides[1]),0.0);
+#endif
     	 break;
        case DOUBLE:
+#ifndef _MSC_VER
 	 n.z = *(double *)((unsigned char*)view->buf+i*view->strides[0]+j*view->strides[1]);
+#else
+	 n.z = _Cbuild(*(double *)((unsigned char*)view->buf+i*view->strides[0]+j*view->strides[1]),0.0);
+#endif
 	 break;
        case COMPLEX:
+#ifndef _MSC_VER
 	 n.z = *(double complex *)((unsigned char*)view->buf+i*view->strides[0]+j*view->strides[1]);
+#else
+	 n.z = *(_Dcomplex *)((unsigned char*)view->buf+i*view->strides[0]+j*view->strides[1]);
+#endif
 	 break;
        }
        MAT_BUFZ(a)[cnt] = n.z;
@@ -293,7 +326,7 @@ Matrix_NewFromNumber(int nrows, int ncols, int id, void *val, int val_id)
 
   int_t i;
   matrix *a = Matrix_New(nrows, ncols, id);
-  if (!a) return (matrix *)PyErr_NoMemory();
+  if (!a) return NULL;
 
   number n;
   if (convert_num[id](&n, val, val_id, 0)) { Py_DECREF(a); return NULL; }
@@ -311,7 +344,6 @@ matrix * Matrix_NewFromSequence(PyObject *x, int id)
   PyObject *seq = PySequence_Fast(x, "list is not iterable");
   if (!seq) return NULL;
 
-
   if (id == -1) {
     for (i=0; i<len; i++) {
       PyObject *item = PySequence_Fast_GET_ITEM(seq, i);
@@ -327,7 +359,7 @@ matrix * Matrix_NewFromSequence(PyObject *x, int id)
   if (!len) { Py_DECREF(seq); return Matrix_New(0, 1, (id < 0 ? INT : id)); }
 
   matrix *L = Matrix_New(len,1,id);
-  if (!L) { Py_DECREF(seq); return (matrix *)PyErr_NoMemory(); }
+  if (!L) { Py_DECREF(seq); return NULL; }
 
   for (i=0; i<len; i++) {
     PyObject *item = PySequence_Fast_GET_ITEM(seq, i);
@@ -354,7 +386,7 @@ matrix * dense(spmatrix *self)
   int_t j, k;
 
   if (!(A = Matrix_New(SP_NROWS(self),SP_NCOLS(self),SP_ID(self))))
-    return (matrix *)PyErr_NoMemory();
+    return NULL;
 
   if (SP_ID(self) == DOUBLE) {
     for (j=0; j<SP_NCOLS(self); j++)
@@ -422,7 +454,7 @@ matrix * dense_concat(PyObject *L, int id_arg)
   id = MAX(id, id_arg);
 
   matrix *A = Matrix_New(m, n, id);
-  if (!A) return (matrix *)PyErr_NoMemory();
+  if (!A) return NULL;
 
   nk = 0;
   for (j=0; j<(single_col ? 1 : PyList_GET_SIZE(L)); j++) {
@@ -523,7 +555,7 @@ matrix_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
   /* sparse matrix */
   else if (SpMatrix_Check(Objx)) {
     matrix *tmp = dense((spmatrix *)Objx);
-    if (!tmp) return PyErr_NoMemory();
+    if (!tmp) return NULL;
     if (tmp->id != id) {
       ret = Matrix_NewFromMatrix(tmp, (id == -1 ? SP_ID(Objx) : id));
       Py_DECREF(tmp);
@@ -655,7 +687,7 @@ matrix * create_indexlist(int_t dim, PyObject *A)
     if ((x = Matrix_New((int)lgt, 1, INT)))
       for (i=start, j=0; j<lgt; i += step, j++) MAT_BUFI(x)[j] = i;
     else {
-      return (matrix *)PyErr_NoMemory();
+      return NULL;
     }
     return x;
   }
@@ -709,7 +741,7 @@ matrix_subscr(matrix* self, PyObject* args)
     int i;
     if (!(ret = Matrix_New(MAT_LGT(Il), 1, self->id) ))
       free_lists_exit(args,(PyObject *)NULL,Il,(PyObject *)NULL,
-          PyErr_NoMemory());
+          NULL);
 
     for (i=0; i<MAT_LGT(Il); i++)
       write_num[self->id](ret->buffer, i, self->buffer,
@@ -756,7 +788,7 @@ matrix_subscr(matrix* self, PyObject* args)
 #endif
 
     if (!(ret = Matrix_New((int)rowlgt, (int)collgt, self->id)))
-      return PyErr_NoMemory();
+      return NULL;
 
     int i, j, icnt, jcnt, cnt=0;
     for (j=colstart, jcnt=0; jcnt<collgt; jcnt++, j+=colstep)
@@ -784,7 +816,7 @@ matrix_subscr(matrix* self, PyObject* args)
 
   int i, j, cnt;
   if (!(ret = Matrix_New(MAT_LGT(Il), MAT_LGT(Jl), self->id)))
-    free_lists_exit(argI, argJ, Il, Jl, PyErr_NoMemory());
+    free_lists_exit(argI, argJ, Il, Jl, NULL);
 
   for (j=0, cnt=0; j < MAT_LGT(Jl); j++)
     for (i=0; i < MAT_LGT(Il); i++) {
@@ -1005,7 +1037,7 @@ static PyMappingMethods matrix_as_mapping = {
 static PyObject * matrix_transpose(matrix *self) {
 
   matrix *ret = Matrix_New(self->ncols, self->nrows, self->id);
-  if (!ret) return PyErr_NoMemory();
+  if (!ret) return NULL;
 
   int i, j, cnt = 0;
   for (i=0; i < ret->nrows; i++)
@@ -1020,7 +1052,7 @@ static PyObject * matrix_ctranspose(matrix *self) {
   if (self->id != COMPLEX) return matrix_transpose(self);
 
   matrix *ret = Matrix_New(self->ncols, self->nrows, self->id);
-  if (!ret) return PyErr_NoMemory();
+  if (!ret) return NULL;
 
   int i, j, cnt = 0;
   for (i=0; i < ret->nrows; i++)
@@ -1036,7 +1068,7 @@ static PyObject * matrix_real(matrix *self) {
     return (PyObject *)Matrix_NewFromMatrix(self,self->id);
 
   matrix *ret = Matrix_New(self->nrows, self->ncols, DOUBLE);
-  if (!ret) return PyErr_NoMemory();
+  if (!ret) return NULL;
 
   int i;
   for (i=0; i < MAT_LGT(self); i++)
@@ -1052,13 +1084,13 @@ static PyObject * matrix_imag(matrix *self) {
     PyObject *a = PyFloat_FromDouble(0);
     ret = Matrix_NewFromNumber(self->nrows, self->ncols, self->id, a, 2);
     Py_DECREF(a);
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     return (PyObject *)ret;
   }
 
   if (!(ret = Matrix_New(self->nrows, self->ncols, DOUBLE)))
-    return PyErr_NoMemory();
+    return NULL;
 
   int i;
   for (i=0; i < MAT_LGT(self); i++)
@@ -1463,7 +1495,7 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
       convert_num[id](&n,self,(Matrix_Check(self) ? 0 : 1),0);
 
       matrix *ret = Matrix_NewFromMatrix((matrix *)other, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(ret); int i;
       switch (id) {
@@ -1474,7 +1506,11 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
           for (i=0; i<lgt; i++) MAT_BUFD(ret)[i] += n.d;
           break;
         case COMPLEX: 
+#ifndef _MSC_VER
           for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] += n.z;
+#else
+          for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] = _Cbuild(creal(n.z)+creal(MAT_BUFZ(ret)[i]), cimag(n.z)+cimag(MAT_BUFZ(ret)[i]));
+#endif
           break;
       }
       return (PyObject *)ret;
@@ -1485,7 +1521,11 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
       switch (id) {
         case INT:     MAT_BUFI(self)[0] += n.i; break;
         case DOUBLE:  MAT_BUFD(self)[0] += n.d; break;
+#ifndef _MSC_VER
         case COMPLEX: MAT_BUFZ(self)[0] += n.z; break;
+#else
+        case COMPLEX: MAT_BUFZ(self)[0] = _Cbuild(creal(n.z)+creal(MAT_BUFZ(self)[0]), cimag(n.z)+cimag(MAT_BUFZ(self)[0])); break;
+#endif
       }
       Py_INCREF(self);
       return self;
@@ -1500,7 +1540,7 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
 
     if (!inplace) {
       matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(self); int i;
       switch (id) {
@@ -1511,7 +1551,11 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
           for (i=0; i<lgt; i++) MAT_BUFD(ret)[i] += n.d;
           break;
         case COMPLEX: 
+#ifndef _MSC_VER
           for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] += n.z;
+#else
+          for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] = _Cbuild(creal(n.z)+creal(MAT_BUFZ(ret)[i]),cimag(n.z)+cimag(MAT_BUFZ(ret)[i]));
+#endif
           break;
       }
 
@@ -1527,7 +1571,11 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
           for (i=0; i<lgt; i++) MAT_BUFD(self)[i] += n.d;
           break;
         case COMPLEX: 
+#ifndef _MSC_VER
           for (i=0; i<lgt; i++) MAT_BUFZ(self)[i] += n.z;
+#else
+          for (i=0; i<lgt; i++) MAT_BUFZ(self)[i] = _Cbuild(creal(n.z)+creal(MAT_BUFZ(self)[i]),cimag(n.z)+cimag(MAT_BUFZ(self)[i]));
+#endif
           break;
       }
 
@@ -1545,7 +1593,7 @@ matrix_add_generic(PyObject *self, PyObject *other, int inplace)
 
     if (!inplace) {
       matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(self), int1 = 1;
       axpy[id](&lgt, &One[id], other_coerce, &int1, MAT_BUF(ret), &int1);
@@ -1601,7 +1649,7 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
       convert_num[id](&n,self,(Matrix_Check(self) ? 0 : 1),0);
 
       matrix *ret = Matrix_NewFromMatrix((matrix *)other, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(ret); int i; 
       switch (id) {
@@ -1612,7 +1660,11 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
           for (i=0; i<lgt; i++) MAT_BUFD(ret)[i] = n.d - MAT_BUFD(ret)[i];
           break;
         case COMPLEX: 
+#ifndef _MSC_VER
           for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] = n.z - MAT_BUFZ(ret)[i];
+#else
+          for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] = _Cbuild(creal(n.z)-creal(MAT_BUFZ(ret)[i]),cimag(n.z)-cimag(MAT_BUFZ(ret)[i]));
+#endif
           break;
       }
       return (PyObject *)ret;
@@ -1623,7 +1675,11 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
       switch (id) {
         case INT:     MAT_BUFI(self)[0] -= n.i; break;
         case DOUBLE:  MAT_BUFD(self)[0] -= n.d; break;
+#ifndef _MSC_VER
         case COMPLEX: MAT_BUFZ(self)[0] -= n.z; break;
+#else
+        case COMPLEX: MAT_BUFZ(self)[0] = _Cbuild(creal(MAT_BUFZ(self)[0])-creal(n.z),cimag(MAT_BUFZ(self)[0])-cimag(n.z)); break;
+#endif
       }
 
       Py_INCREF(self);
@@ -1638,7 +1694,7 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
 
     if (!inplace) {
       matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(self); int i;
       switch (id) {
@@ -1649,7 +1705,11 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
           for (i=0; i<lgt; i++) MAT_BUFD(ret)[i] -= n.d;
           break;
         case COMPLEX: 
+#ifndef _MSC_VER
           for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] -= n.z;
+#else
+          for (i=0; i<lgt; i++) MAT_BUFZ(ret)[i] = _Cbuild(creal(MAT_BUFZ(ret)[i])-creal(n.z),cimag(MAT_BUFZ(ret)[i])-cimag(n.z));
+#endif
           break;
       }
 
@@ -1665,7 +1725,11 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
           for (i=0; i<lgt; i++) MAT_BUFD(self)[i] -= n.d;
           break;
         case COMPLEX: 
+#ifndef _MSC_VER
           for (i=0; i<lgt; i++) MAT_BUFZ(self)[i] -= n.z;
+#else
+          for (i=0; i<lgt; i++) MAT_BUFZ(self)[i] = _Cbuild(creal(MAT_BUFZ(self)[i])-creal(n.z),cimag(MAT_BUFZ(self)[i])-cimag(n.z));
+#endif
           break;
       }
 
@@ -1683,7 +1747,7 @@ matrix_sub_generic(PyObject *self, PyObject *other, int inplace)
 
     if (!inplace) {
       matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(self), int1 = 1;
       axpy[id](&lgt, &MinusOne[id], other_coerce, &int1, MAT_BUF(ret), &int1);
@@ -1739,7 +1803,7 @@ matrix_mul_generic(PyObject *self, PyObject *other, int inplace)
       convert_num[id](&n,self,(Matrix_Check(self) ? 0 : 1),0);
 
       matrix *ret = Matrix_NewFromMatrix((matrix *)other, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(ret), int1 = 1;
       scal[id](&lgt, &n, ret->buffer, &int1);
@@ -1764,7 +1828,7 @@ matrix_mul_generic(PyObject *self, PyObject *other, int inplace)
 
     if (!inplace) {
       matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       int lgt = MAT_LGT(self), int1 = 1;
       scal[id](&lgt, &n, ret->buffer, &int1);
@@ -1802,7 +1866,7 @@ matrix_mul_generic(PyObject *self, PyObject *other, int inplace)
     if (!c) {
       if (MAT_ID(self) != id)  { free(self_coerce); }
       if (MAT_ID(other) != id) { free(other_coerce); }
-      return PyErr_NoMemory();
+      return NULL;
     }
 
     gemm[id](&transA, &transB, &m, &n, &k, &One[id], self_coerce,
@@ -1845,7 +1909,7 @@ matrix_div_generic(PyObject *self, PyObject *other, int inplace)
 
   if (!inplace) {
     matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int lgt = MAT_LGT(ret);
     if (div_array[id](ret->buffer, n, lgt)) { Py_DECREF(ret); return NULL; }
@@ -1891,7 +1955,7 @@ matrix_rem_generic(PyObject *self, PyObject *other, int inplace)
 
   if (!inplace) {
     matrix *ret = Matrix_NewFromMatrix((matrix *)self, id);
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int lgt = MAT_LGT(ret);
     if (mtx_rem[id](ret->buffer, n, lgt)) { Py_DECREF(ret); return NULL; }
@@ -1923,7 +1987,7 @@ static PyObject * matrix_irem(PyObject *self, PyObject *other)
 static PyObject * matrix_neg(matrix *self)
 {
   matrix *x = Matrix_NewFromMatrix(self,self->id);
-  if (!x) return PyErr_NoMemory();
+  if (!x) return NULL;
 
   int n = MAT_LGT(x), int1 = 1;
   scal[x->id](&n, &MinusOne[x->id], x->buffer, &int1);
@@ -1934,7 +1998,7 @@ static PyObject * matrix_neg(matrix *self)
 static PyObject * matrix_pos(matrix *self)
 {
   matrix *x = Matrix_NewFromMatrix(self, self->id);
-  if (!x) return PyErr_NoMemory();
+  if (!x) return NULL;
 
   return (PyObject *)x;
 }
@@ -1944,7 +2008,7 @@ static PyObject * matrix_abs(matrix *self)
   matrix *ret = Matrix_New(self->nrows, self->ncols,
       (self->id == COMPLEX ? DOUBLE : self->id));
 
-  if (!ret) return PyErr_NoMemory();
+  if (!ret) return NULL;
 
   mtx_abs[self->id](MAT_BUF(self), MAT_BUF(ret), MAT_LGT(self));
   return (PyObject *)ret;
@@ -1958,7 +2022,7 @@ static PyObject * matrix_pow(PyObject *self, PyObject *other)
   int id = MAX(DOUBLE, MAX(MAT_ID(self), get_id(other, 1)));
   convert_num[id](&val, other, 1, 0);
   matrix *Y = Matrix_NewFromMatrix((matrix *)self, id);
-  if (!Y) return PyErr_NoMemory();
+  if (!Y) return NULL;
 
   int i;
   for (i=0; i<MAT_LGT(Y); i++) {
@@ -1972,7 +2036,11 @@ static PyObject * matrix_pow(PyObject *self, PyObject *other)
 
       MAT_BUFD(Y)[i] = pow(MAT_BUFD(Y)[i], val.d);
     } else {
+#ifndef _MSC_VER
       if (MAT_BUFZ(Y)[i] == 0.0 && (cimag(val.z) != 0.0 || creal(val.z)<0.0)) {
+#else
+      if ((creal(MAT_BUFZ(Y)[i]) == 0.0 && cimag(MAT_BUFZ(Y)[i]) == 0.0) && (cimag(val.z) != 0.0 || creal(val.z)<0.0)) {
+#endif
         Py_DECREF(Y);
         PY_ERR(PyExc_ValueError, "domain error");
       }
@@ -1989,7 +2057,11 @@ static int matrix_nonzero(matrix *self)
   for (i=0; i<MAT_LGT(self); i++) {
     if ((MAT_ID(self) == INT) && (MAT_BUFI(self)[i] != 0)) res = 1;
     else if ((MAT_ID(self) == DOUBLE) && (MAT_BUFD(self)[i] != 0)) res = 1;
+#ifndef _MSC_VER
     else if ((MAT_ID(self) == COMPLEX) && (MAT_BUFZ(self)[i] != 0)) res = 1;
+#else
+    else if ((MAT_ID(self) == COMPLEX) && (creal(MAT_BUFZ(self)[i]) != 0 || cimag(MAT_BUFZ(self)[i]) != 0)) res = 1;
+#endif
   }
 
   return res;
@@ -2170,7 +2242,11 @@ PyObject * matrix_log(matrix *self, PyObject *args, PyObject *kwrds)
     number n;
     convert_num[COMPLEX](&n, A, 1, 0);
 
+#ifndef _MSC_VER
     if (n.z == 0) PY_ERR(PyExc_ValueError, "domain error");
+#else
+    if (creal(n.z) == 0 && cimag(n.z) == 0) PY_ERR(PyExc_ValueError, "domain error");
+#endif
 
     n.z = clog(n.z);
     return num2PyObject[COMPLEX](&n, 0);
@@ -2192,7 +2268,7 @@ PyObject * matrix_log(matrix *self, PyObject *args, PyObject *kwrds)
 
     if (val > 0.0) {
       matrix *ret = Matrix_New(MAT_NROWS(A), MAT_NCOLS(A), DOUBLE);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       for (i=0; i<MAT_LGT(A); i++)
         MAT_BUFD(ret)[i] = log((MAT_ID(A)== INT ?
@@ -2205,11 +2281,15 @@ PyObject * matrix_log(matrix *self, PyObject *args, PyObject *kwrds)
   }
   else if (Matrix_Check(A) && MAT_ID(A) == COMPLEX) {
     matrix *ret = Matrix_New(MAT_NROWS(A), MAT_NCOLS(A), COMPLEX);
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int i;
     for (i=0; i<MAT_LGT(A); i++) {
+#ifndef _MSC_VER
       if (MAT_BUFZ(A)[i] == 0) {
+#else
+	if (creal(MAT_BUFZ(A)[i]) == 0 && cimag(MAT_BUFZ(A)[i]) == 0) {
+#endif
         Py_DECREF(ret);
         PY_ERR(PyExc_ValueError, "domain error");
       }
@@ -2243,7 +2323,7 @@ PyObject * matrix_exp(matrix *self, PyObject *args, PyObject *kwrds)
   else if (Matrix_Check(A)) {
     matrix *ret = Matrix_New(MAT_NROWS(A),MAT_NCOLS(A),
         (MAT_ID(A) == COMPLEX ? COMPLEX : DOUBLE));
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int i;
     if (MAT_ID(ret) == DOUBLE)
@@ -2297,7 +2377,7 @@ PyObject * matrix_sqrt(matrix *self, PyObject *args, PyObject *kwrds)
 
     if (val >= 0.0) {
       matrix *ret = Matrix_New(MAT_NROWS(A), MAT_NCOLS(A), DOUBLE);
-      if (!ret) return PyErr_NoMemory();
+      if (!ret) return NULL;
 
       for (i=0; i<MAT_LGT(A); i++)
         MAT_BUFD(ret)[i] = sqrt((MAT_ID(A)== INT ?
@@ -2310,7 +2390,7 @@ PyObject * matrix_sqrt(matrix *self, PyObject *args, PyObject *kwrds)
   }
   else if (Matrix_Check(A) && MAT_ID(A) == COMPLEX) {
     matrix *ret = Matrix_New(MAT_NROWS(A), MAT_NCOLS(A), COMPLEX);
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int i;
     for (i=0; i<MAT_LGT(A); i++)
@@ -2344,7 +2424,7 @@ PyObject * matrix_cos(matrix *self, PyObject *args, PyObject *kwrds)
   else if (Matrix_Check(A)) {
     matrix *ret = Matrix_New(MAT_NROWS(A),MAT_NCOLS(A),
         (MAT_ID(A) == COMPLEX ? COMPLEX : DOUBLE));
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int_t i;
     if (MAT_ID(ret) == DOUBLE)
@@ -2383,7 +2463,7 @@ PyObject * matrix_sin(matrix *self, PyObject *args, PyObject *kwrds)
   else if (Matrix_Check(A)) {
     matrix *ret = Matrix_New(MAT_NROWS(A),MAT_NCOLS(A),
         (MAT_ID(A) == COMPLEX ? COMPLEX : DOUBLE));
-    if (!ret) return PyErr_NoMemory();
+    if (!ret) return NULL;
 
     int_t i;
     if (MAT_ID(ret) == DOUBLE)
