@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 M. Andersen and L. Vandenberghe.
+ * Copyright 2012-2019 M. Andersen and L. Vandenberghe.
  * Copyright 2010-2011 L. Vandenberghe.
  * Copyright 2004-2009 J. Dahl and L. Vandenberghe.
  *
@@ -23,9 +23,15 @@
 #include "misc.h"
 #include <fftw3.h>
 
+#ifndef _MSC_VER
+typedef complex double complex_t;
+#else
+typedef _Dcomplex complex_t;
+#endif
+
 PyDoc_STRVAR(fftw__doc__, "Interface to the FFTW3 library.\n");
 
-extern void zscal_(int *n, double complex *alpha, double complex *x, int *incx);
+extern void zscal_(int *n, complex_t *alpha, complex_t *x, int *incx);
 extern void dscal_(int *n, double *alpha, double *x, int *incx);
 
 static char doc_dft[] =
@@ -201,7 +207,11 @@ static PyObject *idft(PyObject *self, PyObject *args, PyObject *kwrds)
   Py_END_ALLOW_THREADS
 
   number a;
+#ifndef _MSC_VER
   a.z = 1.0/m;
+#else
+  a.z = _Cbuild(1.0/m,0.0);
+#endif
   int mn = m*n, ix = 1;
   zscal_(&mn, &a.z, MAT_BUFZ(X), &ix);
 
@@ -307,7 +317,11 @@ static PyObject *idftn(PyObject *self, PyObject *args, PyObject *kwrds)
   }
 
   number a;
+#ifndef _MSC_VER
   a.z = 1.0/proddim;
+#else
+  a.z = _Cbuild(1.0/proddim,0.0);
+#endif
 
   int ix = 1;
   zscal_(&proddim, &a.z, MAT_BUFZ(X), &ix);
